/*
    Copyright (C) 2011 jeanfi@gmail.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301 USA
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <sys/types.h>

#include <json/json.h>

#include "html.h"
#include "lp.h"
#include "lp_ws.h"
#include "ppastats.h"

enum file_copy_error {
	FILE_COPY_ERROR_OPEN_SRC = 1,
	FILE_COPY_ERROR_OPEN_DST,
	FILE_COPY_ERROR_READ,
	FILE_COPY_ERROR_WRITE,
	FILE_COPY_ERROR_ALLOC_BUFFER
};

#define HTML_TEMPLATE \
"<html>\n\
  <head>\n\
    <link type=\"text/css\"\n\
	  rel=\"stylesheet\"\n\
	  href=\n\
\"http://fonts.googleapis.com/css?family=Ubuntu:regular,bold&subset=Latin\">\n\
    <link type=\"text/css\" href=\"css/ppastats.css\" rel=\"stylesheet\" />\n\
    <link type=\"text/css\" href=\"css/jquery.jqplot.min.css\"\n\
	  rel=\"stylesheet\" />\n\
    <script type=\"text/javascript\" src=\"js/jquery.min.js\"></script>\n\
    <script type=\"text/javascript\"\n\
	    src=\"js/jquery.jqplot.min.js\"></script>\n\
    <script type=\"text/javascript\"\n\
	    src=\"js/jqplot.dateAxisRenderer.min.js\"></script>\n\
    <script>var data = %s;</script>\n\
    <script type=\"text/javascript\" src=\"js/ppastats.js\"></script>\n\
  </head>\n\
  <body>\n\
    <h1><span id=\"pkg_name\">N/A</span></h1>\n\
    <p><em>PPA</em>: \n\
       <a href=\".\">\n\
	   <span id=\"ppa_owner\">N/A</span>/<span id=\"ppa_name\">N/A</span>\n\
       </a></p>\n\
    <div id=\"chart\"></div>\n\
  </body>\n\
</html>"

#define FCOPY_BUF_SZ 4096
static int file_copy(FILE * src, FILE * dst)
{
	int ret = 0;
	char *buf = malloc(FCOPY_BUF_SZ);
	int n;

	if (!buf)
		return FILE_COPY_ERROR_ALLOC_BUFFER;

	while (!ret) {
		n = fread(buf, 1, FCOPY_BUF_SZ, src);
		if (n) {
			if (fwrite(buf, 1, n, dst) != n)
				ret = FILE_COPY_ERROR_WRITE;
		} else {
			if (!feof(src))
				ret = FILE_COPY_ERROR_READ;
			else
				break;
		}
	}

	free(buf);

	return ret;
}

int
fcopy(const char *src, const char *dst)
{
	FILE *fsrc, *fdst;
	int ret = 0;

	if (debug)
		printf("DEBUG: copy: %s to %s\n", src, dst);

	fsrc = fopen(src, "r");

	if (fsrc) {
		fdst = fopen(dst, "w+");

		if (fdst) {
			ret = file_copy(fsrc, fdst);
			fclose(fdst);
		} else {
			ret = FILE_COPY_ERROR_OPEN_DST;
		}

		fclose(fsrc);
	} else {
		ret = FILE_COPY_ERROR_OPEN_SRC;
	}

	return ret;
}

static char *get_path(const char *dir, const char *file)
{
	char *path = malloc(strlen(dir)+1+
			    strlen(file)+
			    strlen(".html")+
			    1);

	strcpy(path, dir);
	strcat(path, "/");
	strcat(path, file);
	strcat(path, ".html");

	return path;
}

static struct json_object *date_to_json(struct tm *tm)
{
	json_object *json;

	json = json_object_new_array();
	json_object_array_add(json, json_object_new_int(tm->tm_year+1900));
	json_object_array_add(json, json_object_new_int(tm->tm_mon+1));
	json_object_array_add(json, json_object_new_int(tm->tm_mday));

	return json;
}

static char *pkg_to_json(struct ppa_stats *ppa, struct package_stats *pkg)
{
	json_object *json, *json_ddts, *json_ddt;
	struct daily_download_total **ddts, *ddt;
	char *ret;

	json = json_object_new_object();

	json_object_object_add(json,
			       "ppa_name", json_object_new_string(ppa->name));
	json_object_object_add(json,
			       "ppa_owner",
			       json_object_new_string(ppa->owner));

	json_object_object_add(json,
			       "name", json_object_new_string(pkg->name));

	json_ddts = json_object_new_array();
	json_object_object_add(json, "ddts", json_ddts);

	ddts = pkg->daily_download_totals;
	while (*ddts) {
		ddt = *ddts;

		json_ddt = json_object_new_object();
		json_object_object_add(json_ddt,
				       "value",
				       json_object_new_int(ddt->count));
		json_object_object_add(json_ddt,
				       "time",
				       date_to_json(&ddt->date));

		json_object_array_add(json_ddts, json_ddt);

		ddts++;
	}

	ret = strdup(json_object_to_json_string(json));

	json_object_put(json);

	return ret;
}

static void
pkg_to_html(struct ppa_stats *ppa, struct package_stats *pkg, const char *dir)
{
	char *path;
	FILE *f;
	char *json;

	path = get_path(dir, pkg->name);
	f = fopen(path, "w");

	if (!f) {
		fprintf(stderr, "ERROR: failed to open: %s\n", path);
		return ;
	}

	json = pkg_to_json(ppa, pkg);

	fprintf(f, HTML_TEMPLATE, json);

	fclose(f);

	free(path);
	free(json);
}

static void
packages_to_html(struct ppa_stats *ppa,
		 struct package_stats **packages,
		 const char *dir)
{
	struct package_stats **cur;

	cur = packages;
	while (*cur) {
		pkg_to_html(ppa, *cur, dir);
		cur++;
	}
}

static char *append_path(const char *odir, const char *name)
{
	char *dir;

	dir = malloc(strlen(odir)+1+strlen(name)+1);

	sprintf(dir, "%s/%s", odir, name);

	return dir;
}

void
ppa_to_html(const char *owner,
	    const char *ppa,
	    const char *package_status,
	    const char *output_dir)
{
	struct ppa_stats *ppastats;
	struct daily_download_total **totals;
	char *path, *f_dst;
	char *css_dir, *js_dir;
	int i;
	static char *www_files[]
		= { DEFAULT_WWW_DIR"/jquery.min.js", "js/jquery.min.js",
		    DEFAULT_WWW_DIR"/ppastats.js", "js/ppastats.js",
		    DEFAULT_WWW_DIR"/jqplot.dateAxisRenderer.min.js",
		    "js/jqplot.dateAxisRenderer.min.js",
		    DEFAULT_WWW_DIR"/jquery.jqplot.min.js",
		    "js/jquery.jqplot.min.js",
		    DEFAULT_WWW_DIR"/ppastats.css", "css/ppastats.css",
		    DEFAULT_WWW_DIR"/jquery.jqplot.min.css",
		    "css/jquery.jqplot.min.css" };

	css_dir = append_path(output_dir, "css");
	js_dir = append_path(output_dir, "js");

	mkdir(css_dir, 0777);
	mkdir(js_dir, 0777);

	for (i = 0; i < 6; i++) {
		f_dst = append_path(output_dir, www_files[2*i+1]);
		fcopy(www_files[2*i], f_dst);

		free(f_dst);
	}

	ppastats = create_ppa_stats(owner, ppa, package_status);
	totals = ppastats->daily_download_totals;

	path = get_path(output_dir, "ppa");

	packages_to_html(ppastats, ppastats->packages, output_dir);

	ppa_stats_free(ppastats);

	free(path);
	free(css_dir);
	free(js_dir);
}
